﻿using LibUsbDotNet;
using LibUsbDotNet.Main;
using System.ComponentModel;

namespace TI83PlusBootExec {

	[Description("Silver Link (WinUSB driver)")]
	internal class SilverLinkWinUsb : LinkCable {

		UsbDevice usbDevice;
		UsbEndpointReader usbReader;
		UsbEndpointWriter usbWriter;

		public SilverLinkWinUsb() {
			this.usbDevice = UsbDevice.OpenUsbDevice(new UsbDeviceFinder(0x0451, 0xE001));

			if (this.usbDevice != null) {
				this.usbWriter = this.usbDevice.OpenEndpointWriter(WriteEndpointID.Ep02);
				this.usbWriter.Reset();

				this.usbReader = this.usbDevice.OpenEndpointReader(ReadEndpointID.Ep01);
				this.usbReader.Reset();
			}
		}

		public bool IsOpen {
			get { return this.usbDevice != null; }
		}

		public override bool TryReadByte(out byte value) {

			var buffer = new byte[1];
			this.usbReader.Read(buffer, 1000, out int read);

			if (read > 0) {
				value = buffer[0];
				return true;
			} else {
				value = 0;
				return false;
			}

		}

		public override bool TryWriteByte(byte value) {
			var buffer = new byte[] { value };
			this.usbWriter.Write(buffer, 1000, out int written);
			return written == buffer.Length;
		}

		public override bool TryWriteBytes(byte[] values) {
			this.usbWriter.Write(values, 1000, out int written);
			return written == values.Length;
		}
	}
}
